//========================================================================================
//	Release Version:	$ ODF 1 $
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.

//================================================================================
#include "DataSave.hpp"

#ifndef PART_H
#include "Part.h"
#endif

#ifndef CONTENT_H
#include "Content.h"
#endif

#ifndef BINDING_K
#include "Binding.k"
#endif

#ifndef FRAME_H
#include "Frame.h"
#endif

// ----- Framework Layer -----
#ifndef FWPRESEN_H
#include "FWPresen.h"		// FW_CPresentation
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"			// FW_Beep()
#endif

#ifndef FWABOUT_H
#include "FWAbout.h"		//::FW_About()
#endif

// ----- OS Layer -----
#ifndef FWMENU_H
#include "FWMenu.h"			// FW_CMenuBar, FW_CPullDownMenu
#endif

#ifndef FWCFMRES_H
#include "FWCFMRes.h"		// FW_CSharedLibraryResourceFile
#endif

#ifndef FWRESTYP_H
#include "FWResTyp.h"		// MULTISTRINGRES
#endif

#ifndef FWEVENT_H
#include "FWEvent.h"		// FW_CMenuEvent
#endif

#ifndef FWSUSINK_H
#include "FWSUSink.h"		// FW_CStorageUnitSink
#endif

#ifndef FWBARRAY_H
#include "FWBArray.h"		// FW_CByteArray
#endif

// ----- Foundation Layer -----
#ifndef FWSTREAM_H
#include <FWStream.h>		// FW_InitializeArchiving
#endif

#ifndef FWMEMMGR_H
#include "FWMemMgr.h"		// FW_CMemoryManager
#endif

#ifndef FWSUSINK_H
#include "FWSUSink.h"		// FW_CStorageUnitSink
#endif

//--- OpenDoc ------------------------------------------------------------------
#ifndef SOM_Module_OpenDoc_StdProps_defined
#include <StdProps.xh>		// kODPropContents
#endif

//==============================================================================
#ifdef FW_BUILD_MAC
#pragma segment DataSave
#endif

FW_DEFINE_AUTO(CDataSavePart)

//==============================================================================
CDataSavePart::CDataSavePart(ODPart* odPart)
  :	FW_CPart(odPart, FW_gInstance, kPartInfoID),
	fPresentation(NULL),
	fPartContent(NULL)
{
}

//--------------------------------------------------------------------------------
CDataSavePart::~CDataSavePart()
{
}

//--------------------------------------------------------------------------------
void 
CDataSavePart::Initialize(Environment* ev)	// Override
{
	FW_CPart::Initialize(ev);
	FW_CSelection* selection = NULL;
	const ODType kMainPresentation = "Apple:Presentation:DataSave";
	const FW_Boolean kDefaultPresentation = true;
	fPresentation = RegisterPresentation(ev, kMainPresentation, 
											kDefaultPresentation, selection);
}

//--------------------------------------------------------------------------------
FW_Boolean 
CDataSavePart::DoMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent)	// Override
{
	FW_Boolean menuHandled = TRUE;
	ODCommandID commandID = theMenuEvent.GetCommandID(ev);
	
	switch (commandID)
	{
		case kODCommandAbout:
			::FW_About(ev, this, kAbout);
			break;

		default:
			menuHandled = FALSE;
	}
	
	return menuHandled;
}

//--------------------------------------------------------------------------------
FW_CFrame* 
CDataSavePart::NewFrame(Environment* ev, ODFrame* odFrame,
						FW_CPresentation* presentation, FW_Boolean fromStorage)	// Override
{
	FW_UNUSED(presentation);
	FW_UNUSED(fromStorage);
	return FW_NEW(CDataSaveFrame, (ev, odFrame, presentation, fPartContent));
}

//------------------------------------------------------------------------------
FW_CContent* 
CDataSavePart::NewPartContent(Environment* ev)
{
	fPartContent = FW_NEW(CDataSaveContent, (ev, this));
	return fPartContent;
}

//------------------------------------------------------------------------------
void 
CDataSavePart::MyInvalidatePresentation(Environment* ev, FW_CRect& rect)
{
	fPresentation->Invalidate(ev, rect);
}
